package websocket;

import dbnotification.DBChangeNotification;

import java.io.IOException;

import java.util.Queue;
import java.util.concurrent.ConcurrentLinkedQueue;

import javax.websocket.OnClose;
import javax.websocket.OnError;
import javax.websocket.OnMessage;
import javax.websocket.OnOpen;
import javax.websocket.Session;
import javax.websocket.server.ServerEndpoint;

@ServerEndpoint(value = "/service")
public class Service {

    public Service() {
        super();
    }

    final static Queue<Session> queue = new ConcurrentLinkedQueue<>();

    @OnOpen
    public void open(Session session) {
        queue.add(session);
        System.out.println("***** WebSocket: connection has been OPENED");

        try {
            (new DBChangeNotification()).subscribeForDBChanges();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @OnClose
    public void closedConnection(Session session) {
        queue.remove(session);
        System.out.println("***** WebSocket: connection has been CLOSED");
    }

    @OnError
    public void error(Session session, Throwable t) {
        queue.remove(session);
        System.out.println("***** WebSocket: Exception: ");
        t.printStackTrace();
    }

    @OnMessage
    public void processMessageFromClient(Session session, String message) {
        System.out.println("***** WebSocket: processing message from client");
    }


    public static void SendMessageToClient(String message) {

        for (Session session : queue) {
            try {
                System.out.println("***** WebSocket: notifying client session: " + session.getId());
                session.getBasicRemote().sendText(message);
            } catch (IOException e) {
                throw new IllegalStateException(e);
            }
        }
    }
}
